<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;
use Drupal\simple_interactive_maps\ClippableContent;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of interactive maps.
 */
final class InteractiveMapListBuilder extends ConfigEntityListBuilder {

  public function __construct(
    EntityTypeInterface $entity_type,
    EntityStorageInterface $storage,
    protected ClippableContent $clippableContent,
    protected MapDefinitionPluginManager $mapPluginManager,
  ) {
    parent::__construct($entity_type, $storage);
  }

  /**
   * {@inheritDoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    /** @var \Drupal\simple_interactive_maps\ClippableContent $clippableContent */
    $clippableContent = $container->get('simple_interactive_maps.clippable_content');

    /** @var \Drupal\simple_interactive_maps\MapDefinitionPluginManager $mapPluginManager */
    $mapPluginManager = $container->get('plugin.manager.map_definition');

    return new static($entity_type, $container->get('entity_type.manager')
      ->getStorage('interactive_map'), $clippableContent, $mapPluginManager);
  }

  /**
   * {@inheritDoc}
   */
  public function buildHeader(): array {
    $header['label'] = $this->t('Label');
    $header['base'] = $this->t('Base map');
    $header['embed'] = $this->t('Embed code');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritDoc}
   */
  public function buildRow(EntityInterface $entity): array {

    assert($entity instanceof InteractiveMapInterface);

    $plugin = $this->mapPluginManager->createInstance($entity->get('base_map'));
    $pluginDefinition = $plugin->getPluginDefinition();

    /** @var \Drupal\simple_interactive_maps\InteractiveMapInterface $entity */
    $row['label'] = $entity->label();
    $row['base'] = $pluginDefinition['label'];
    $row['embed'] = [
      'data' => $this->clippableContent->createClippableItem($entity->getEmbedCode(), 'simple_interactive_maps_embed_code'),
    ];
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritDoc}
   */
  public function getDefaultOperations(EntityInterface $entity): array {
    $operations = parent::getDefaultOperations($entity);

    $operations['details'] = [
      'title' => $this->t('Edit Regions'),
      'weight' => 10,
      'url' => Url::fromRoute('simple_interactive_maps.map_regions_form', ['interactive_map' => $entity->id()]),
    ];

    $operations['groups'] = [
      'title' => $this->t('Edit Groups'),
      'weight' => 20,
      'url' => Url::fromRoute('simple_interactive_maps.map_groups_form', ['interactive_map' => $entity->id()]),
    ];

    $operations['preview'] = [
      'title' => $this->t('Preview'),
      'weight' => 30,
      'url' => Url::fromRoute('simple_interactive_maps.map_preview', ['interactive_map' => $entity->id()]),
    ];

    $operations['tools'] = [
      'title' => $this->t('Tools'),
      'weight' => 40,
      'url' => Url::fromRoute('simple_interactive_maps.simple_interactive_maps_tools', ['interactive_map' => $entity->id()]),
    ];

    return $operations;
  }

}
